@@ -1,8 +1,22 @@
-// AssemblyInfo.cs created with MonoDevelop
-// User: dave at 8:25 PM 8/16/2007
+// AssemblyInfo.cs
 //
-// To change standard headers go to Edit->Preferences->Coding->Standard Headers
+// GNOME Do is the legal property of its developers. Please refer to the
+// COPYRIGHT file distributed with this source distribution.
 //
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
@@ -26,11 +40,21 @@ using System.Runtime.CompilerServices;
 //
 // You can specify all values by your own or you can build default build and revision
 // numbers with the '*' character (the default):
-[assembly: AssemblyVersion("0.6.0.0")]
-[assembly: Do.ProgramVersion("0.6.1.0", "Release")]
+[assembly: AssemblyVersion("0.9.0.0")]
 
 // The following attributes specify the key for the sign of your assembly. See the
 // .NET Framework documentation for more information about signing.
 // This is not required, if you don't want signing let these attributes like they're.
 [assembly: AssemblyDelaySign(false)]
 [assembly: AssemblyKeyFile("")]
+
+namespace Do
+{
+	internal static class AssemblyInfo
+	{
+		public const string DisplayVersion = "0.8.1.3";
+		public const string VersionDetails = "bzr do r1102";
+		public const string LocaleDirectory = "/usr/share/locale";
+	}
+}
+
